#include "displaylfrconfig.h"

DisplayLFRConfig::DisplayLFRConfig(QWidget *parent) :
    QWidget(parent)
{
    defaultStorageDirectory = QDir::homePath();

    initDisplay();

    setLabelFrames();
}

void DisplayLFRConfig::initDisplay()
{
    layout_widgetLayout       = new QVBoxLayout();
    layout_mainLayout         = new QGridLayout();
    groupbox_displayLFRConfig = new QGroupBox(tr("LFR inputs configuration"));

    groupbox_displayLFRConfig->setToolTip( "you need to send a TC_LFR_DUMP_PAR packet and wait for the TM_LFR_PARAMETER_DUMP answer" );

    defaultConfig = "x";

    label_d[0].setText( defaultConfig );
    label_d[1].setText( defaultConfig );
    label_d[2].setText( defaultConfig );
    label_d[3].setText( defaultConfig );
    label_d[4].setText( defaultConfig );

    label_s[0].setText( defaultConfig );
    label_s[1].setText( defaultConfig );
    label_s[2].setText( defaultConfig );
    label_s[3].setText( defaultConfig );
    label_s[4].setText( defaultConfig );

    label_f[0].setText("F0");
    label_f[1].setText("F1");
    label_f[2].setText("F2");
    label_f[3].setText("F3");

    label_v[0].setText("V");
    label_v[1].setText( defaultConfig );
    label_v[2].setText( defaultConfig );
    label_v[3].setText( defaultConfig );
    label_v[4].setText( defaultConfig );

    label_e1[0].setText("E1");
    label_e1[1].setText( defaultConfig );
    label_e1[2].setText( defaultConfig );
    label_e1[3].setText( defaultConfig );
    label_e1[4].setText( defaultConfig );

    label_e2[0].setText("E2");
    label_e2[1].setText( defaultConfig );
    label_e2[2].setText( defaultConfig );
    label_e2[3].setText( defaultConfig );
    label_e2[4].setText( defaultConfig );

    layout_mainLayout->addWidget( &label_f[0], 0, 1, 1, 1);
    layout_mainLayout->addWidget( &label_f[1], 0, 2, 1, 1);
    layout_mainLayout->addWidget( &label_f[2], 0, 3, 1, 1);
    layout_mainLayout->addWidget( &label_f[3], 0, 4, 1, 1);

    layout_mainLayout->addWidget( &label_v[0], 1, 0, 1, 1);
    layout_mainLayout->addWidget( &label_v[1], 1, 1, 1, 1);
    layout_mainLayout->addWidget( &label_v[2], 1, 2, 1, 1);
    layout_mainLayout->addWidget( &label_v[3], 1, 3, 1, 1);
    layout_mainLayout->addWidget( &label_v[4], 1, 4, 1, 1);

    layout_mainLayout->addWidget( &label_e1[0], 2, 0, 1, 1);
    layout_mainLayout->addWidget( &label_e1[1], 2, 1, 1, 1);
    layout_mainLayout->addWidget( &label_e1[2], 2, 2, 1, 1);
    layout_mainLayout->addWidget( &label_e1[3], 2, 3, 1, 1);
    layout_mainLayout->addWidget( &label_e1[4], 2, 4, 1, 1);

    layout_mainLayout->addWidget( &label_e2[0], 3, 0, 1, 1);
    layout_mainLayout->addWidget( &label_e2[1], 3, 1, 1, 1);
    layout_mainLayout->addWidget( &label_e2[2], 3, 2, 1, 1);
    layout_mainLayout->addWidget( &label_e2[3], 3, 3, 1, 1);
    layout_mainLayout->addWidget( &label_e2[4], 3, 4, 1, 1);

    layout_mainLayout->setRowStretch( 4, 1 );
    layout_mainLayout->setColumnStretch( 5, 1 );

    groupbox_displayLFRConfig->setLayout( layout_mainLayout );

    layout_widgetLayout->addWidget( groupbox_displayLFRConfig );

    this->setLayout( layout_widgetLayout );
}

void DisplayLFRConfig::setLabelFrames()
{
    for (unsigned int i = 0; i < 5; i++)
    {
        label_v[i].setFrameStyle(QFrame::Panel | QFrame::Raised);
        label_v[i].setLineWidth(2);

        label_e1[i].setFrameStyle(QFrame::Panel | QFrame::Raised);
        label_e1[i].setLineWidth(2);

        label_e2[i].setFrameStyle(QFrame::Panel | QFrame::Raised);
        label_e2[i].setLineWidth(2);
    }

    for (unsigned int i = 0; i < 5; i++)
    {
        label_f[i].setFrameStyle(QFrame::Panel | QFrame::Raised);
        label_f[i].setLineWidth(2);
    }

}

void DisplayLFRConfig::processCommonParameters( unsigned char sy_lfr_common_parameters )
{
    bool BW;
    bool SP0;
    bool SP1;
    bool R0;
    bool R1;
    bool R2;

    BW  = (bool) ( (sy_lfr_common_parameters & 0x20) >> 5);
    SP0 = (bool) ( (sy_lfr_common_parameters & 0x10) >> 4);
    SP1 = (bool) ( (sy_lfr_common_parameters & 0x08) >> 3);
    R0  = (bool) ( (sy_lfr_common_parameters & 0x04) >> 2);
    R1  = (bool) ( (sy_lfr_common_parameters & 0x02) >> 1);
    R2  = (bool) ( (sy_lfr_common_parameters & 0x01)     );

    updateDisplay( BW, SP0, SP1, R0, R1, R2 );
}

void DisplayLFRConfig::updateDisplay(bool BW, bool SP0, bool SP1, bool R0, bool R1, bool R2)
{
    updateD( BW );
    updateS( SP0, SP1 );
    updateF0( R0 );
    updateF1( R1 );
    updateF2( R2 );
    updateF3();
}

void DisplayLFRConfig::updateD( bool BW )
{
    switch ( BW )
    {
    case true:
        label_d[0].setText( "BIAS_1" );
        label_d[1].setText( "BIAS_2" );
        label_d[2].setText( "BIAS_3" );
        label_d[3].setText( "BIAS_4" );
        label_d[4].setText( "BIAS_5" );
        break;
    case false:
        label_d[0].setText( "VHF_1" );
        label_d[1].setText( "VHF_2" );
        label_d[2].setText( "VHF_3" );
        label_d[3].setText( "GND" );
        label_d[4].setText( "GND" );
        break;
    }
}

void DisplayLFRConfig::updateS( bool SP0,  bool SP1 )
{
    label_s[0].setText( label_d[0].text() );
    label_s[3].setText( label_d[3].text() );
    label_s[4].setText( label_d[4].text() );

    switch (SP0)
    {
    case true:
        label_s[1].setText( label_d[1].text() + " - " + label_d[0].text() );
        break;
    case false:
        label_s[1].setText( label_d[1].text() );
        break;
    }

    switch (SP1)
    {
    case true:
        label_s[2].setText( label_d[2].text() + " - " + label_d[0].text() );
        break;
    case false:
        label_s[2].setText( label_d[2].text() );
        break;
    }
}

void DisplayLFRConfig::updateF0( bool R0 )
{
    label_v[1].setText( label_s[0].text() );

    switch ( R0 )
    {
    case true:
        label_e1[1].setText( label_s[1].text() );
        label_e2[1].setText( label_s[2].text() );
        break;
    case false:
        label_e1[1].setText( label_s[3].text() );
        label_e2[1].setText( label_s[4].text() );
        break;
    }
}

void DisplayLFRConfig::updateF1( bool R1 )
{
    label_v[2].setText( label_s[0].text() );

    switch (R1)
    {
    case true:
        label_e1[2].setText( label_s[1].text() );
        label_e2[2].setText( label_s[2].text() );
        break;
    case false:
        label_e1[2].setText( label_s[3].text() );
        label_e2[2].setText( label_s[4].text() );
        break;
    }
}

void DisplayLFRConfig::updateF2( bool R2 )
{
    label_v[3].setText( label_s[0].text() );

    switch (R2)
    {
    case true:
        label_e1[3].setText( label_s[1].text() );
        label_e2[3].setText( label_s[2].text() );
        break;
    case false:
        label_e1[3].setText( label_s[3].text() );
        label_e2[3].setText( label_s[4].text() );
        break;
    }
}

void DisplayLFRConfig::updateF3( )
{
    label_v[4].setText( label_s[0].text() );
    label_e1[4].setText( label_s[1].text() );
    label_e2[4].setText( label_s[2].text() );
}

void DisplayLFRConfig::setDefaultStorageDirectory(QString nameOfTheDirectory)
{
    this->defaultStorageDirectory = nameOfTheDirectory;
}
