#include "mainwindow.h"
#include <iostream>
#include <QNetworkInterface>
#include <math.h>

MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent)
{
    parsingContinue = false;
    totalOfBytes = 0;
    totalOfPackets = 0;

    spectraPacketNormalSpectrumF0 = new SpectraPacket(0, 128);
    spectraPacketNormalSpectrumF1 = new SpectraPacket(0, 128);
    spectraPacketNormalSpectrumF2 = new SpectraPacket(0, 128);

    //*********
    // wfPacket
    wfPacketNormal[0] = new WFPacket(0, 2048);
    wfPacketNormal[1] = new WFPacket(0, 2048);
    wfPacketNormal[2] = new WFPacket(0, 2048);
    wfPacketNormal[3] = new WFPacket(0, 2688);
    wfPacketBurst = new WFPacket(0, 2688);
    wfPacketSBM1 = new WFPacket(0, 2688);
    wfPacketSBM2 = new WFPacket(0, 2688);
    wfPacketCWF_F3 = new WFPacket(0, 2688);

    //****
    // XML
    // xml handlers
    xmlHandler = new LFRXmlHandler();
    // xml sources
    sourceTC = new QXmlInputSource();
    // xml writer
    lfrXmlWriter = new LFRXmlWriter();
    // setup xml parser for the echo bridge
    lfrXmlParser = new LFRXmlParser();

    UI = new MainWindowUI();
    readSettings();
    this->UI->serverTMEchoDialogBox->setIP(echoBridgeIP0, echoBridgeIP1, echoBridgeIP2, echoBridgeIP3);

    lfrXmlParser->generalCCSDSPacketStore = &this->generalCCSDSPacketStore;

    initSocketStatesList();

    this->setCentralWidget(this->UI);
    //this->setLayout(UI->layout());

    analyserSGSEServerTC = new QTcpServer();
    analyserSGSEServerTM = new QTcpServer();

    socketTC = NULL;
    socketTM = NULL;
    socketEchoServer = new QTcpSocket();

    connect(this->UI->button_openServerTCTM, SIGNAL(clicked()), this, SLOT(listenOnTCTMPorts()));
    connect(this->UI->button_testServerTCTM, SIGNAL(clicked()), this, SLOT(testTCTMPorts()));
    connect(this->UI->button_openSocketEchoServer, SIGNAL(clicked()), this, SLOT(openEchoServer()));
    //
    connect(this->analyserSGSEServerTC, SIGNAL(newConnection()), this, SLOT(newConnectionOnTCServer()));
    connect(this->analyserSGSEServerTM, SIGNAL(newConnection()), this, SLOT(newConnectionOnTMServer()));
    //
    connect(this, SIGNAL(socketTMHasChanged(QTcpSocket*)), this->lfrXmlWriter, SLOT(setSocketTM(QTcpSocket*)));
    connect(this->UI->button_TCAcknowledgement, SIGNAL(clicked()), this->lfrXmlWriter, SLOT(sendXML_TC_Acknowledgment()));
    connect(this->UI->button_TCRejection, SIGNAL(clicked()), this->lfrXmlWriter, SLOT(sendXML_TC_rejection()));
    connect(this->UI->button_GSEHK, SIGNAL(clicked()), this->lfrXmlWriter, SLOT(sendXML_GSE_HK()));

    // socket echo server
    connect(this->socketEchoServer, SIGNAL(stateChanged(QAbstractSocket::SocketState)),
            this, SLOT(socket_TMEcho_ServerHasChanged()));
    connect(this->socketEchoServer, SIGNAL(readyRead()),
            this, SLOT(readDataOnSocketEchoServer()));

    // display on console
    connect(this->xmlHandler, SIGNAL(displayMessage(QString)),
            this, SLOT(displayOnConsole(QString)));
    connect(this->UI->hkDisplay, SIGNAL(displayMessage(QString)),
            this, SLOT(displayOnConsole(QString)));
    connect(this->lfrXmlParser, SIGNAL(sendMessage(QString)),
            this, SLOT(displayOnConsole(QString)));
    //
    connect(this->lfrXmlParser, SIGNAL(processPacketStore()),
            this, SLOT(processPacketStore()));
    //
    connect(this->UI->button_resetStatistics, SIGNAL(clicked()),
            this, SLOT(resetStatistics()));

    displayNetworkInterfaces();

    this->xmlHandler->packetStoreHasChanged(&generalCCSDSPacketStore);
};

MainWindow::~MainWindow()
{
    delete wfPacketNormal[0];
    delete wfPacketNormal[1];
    delete wfPacketNormal[2];
    delete wfPacketNormal[3];
    delete wfPacketBurst;
    delete wfPacketSBM1;
    delete wfPacketSBM2;
    delete wfPacketCWF_F3;
}

void MainWindow::displayNetworkInterfaces()
{
    QList<QHostAddress> list = QNetworkInterface::allAddresses();
    this->displayOnConsole("Network interfaces:");
    for(int nIter=0; nIter<list.count(); nIter++)
    {
        this->UI->displayOnConsole(list[nIter].toString());
    }
}

void MainWindow::listenOnTCTMPorts()
{
    QString str;

    //***
    // TC
    analyserSGSEServerTC->listen(QHostAddress::Any, this->UI->getTCPort());
    str = analyserSGSEServerTC->errorString();
    if (!str.isEmpty())
    {
        this->displayOnConsole(str);
    }
    this->displayOnConsole("Listening TC on port "
                           + QString::number(this->UI->getTCPort())
                           );

    //***
    // TM
    analyserSGSEServerTM->listen(QHostAddress::Any, this->UI->getTMPort());
    str = analyserSGSEServerTM->errorString();
    if (!str.isEmpty())
    {
        this->displayOnConsole(str);
    }
    this->displayOnConsole("Listening TM on port "
                           + QString::number(this->UI->getTMPort())
                           );
}

void MainWindow::testTCTMPorts()
{
    if (socketTC != NULL)
    {
        this->displayOnConsole("TC socket *** " + socketStates.at(socketTC->state()) );
    }
    else
    {
        this->displayOnConsole("TC socket *** not tested, (socketTC) is NULL" );
    }
    if (socketTM != NULL)
    {
        this->displayOnConsole("TM socket *** " + socketStates.at(socketTM->state()) );
    }
    else
    {
        this->displayOnConsole("TM socket *** not tested, (socketTM) is NULL" );
    }
}

void MainWindow::newConnectionOnTCServer()
{
    this->displayOnConsole("got new connection on TC port");
    socketTC = analyserSGSEServerTC->nextPendingConnection();
    this->displayOnConsole("TC socket *** " + socketStates.at(socketTC->state()) );
    connect(this->socketTC, SIGNAL(readyRead()), this, SLOT(readDataOnTCPort()));
}

void MainWindow::newConnectionOnTMServer()
{
    this->displayOnConsole("got new connection on TM port");
    socketTM = analyserSGSEServerTM->nextPendingConnection();
    this->displayOnConsole("TM socket *** " + socketStates.at(socketTM->state()) );
    connect(this->socketTM, SIGNAL(readyRead()), this, SLOT(readDataOnTMPort()));

    emit socketTMHasChanged(this->socketTM);
}

void MainWindow::readDataOnTCPort()
{
    bool ok;
    int nbBytesAvailable = 0;

    nbBytesAvailable = socketTC->bytesAvailable();
    buffer = (char *) malloc(nbBytesAvailable);
    socketTC->read(buffer, nbBytesAvailable);

    this->displayOnConsole("readDataOnTCPort *** "
                           + QString::number(nbBytesAvailable)
                           + " read");

    QByteArray xmlData( buffer, nbBytesAvailable);

    free(buffer);

    sourceTC->setData( xmlData );

    xmlReader.setContentHandler(xmlHandler);
    xmlReader.setErrorHandler(xmlHandler);

    ok = xmlReader.parse(sourceTC);
    if (!ok) {
        std::cout << "Parsing failed." << std::endl;
    }
    else {
        QStringList names = xmlHandler->names();
        QList<int> indentations = xmlHandler->indentations();

        int items = names.count();

        for (int i = 0; i < items; ++i) {
            displayOnConsole(
                        names[i]
                        + " --- identations: "
                        + QString::number(indentations[i]));
        }
    }
}

void MainWindow::readDataOnTMPort()
{
    this->displayOnConsole("TM data received, begin parsing");
}

void MainWindow::readDataOnSocketEchoServer()
{
    QString dataString;

    dataArray.append(socketEchoServer->readAll());
//    dataString = QString::fromAscii(dataArray);
    dataString = QString::fromLatin1(dataArray);
    this->lfrXmlParser->processIncomingStr(dataString);
    dataArray.clear();
}

void MainWindow::displayOnConsole(QString message)
{
    this->UI->displayOnConsole( message );
}

void MainWindow::initSocketStatesList()
{
    socketStates.append("The socket is not connected");
    socketStates.append("The socket is performing a host name lookup");
    socketStates.append("The socket has started establishing a connection");
    socketStates.append("A connection is established");
    socketStates.append("The socket is bound to an address and port (for servers)");
    socketStates.append("The socket is about to close (data may still be waiting to be written)");
    socketStates.append("For internal use only");
}

void MainWindow::openEchoServer()
{
    socketEchoServer->connectToHost(this->UI->serverTMEchoDialogBox->getIP(),
                                    this->UI->spinbox_serverTMEchoPort->value());
    //socketEchoServer->open();
}

void MainWindow::socket_TMEcho_ServerHasChanged()
{
    this->displayOnConsole("TM Echo Socket socket *** " + socketStates.at(socketEchoServer->state()) );
}

void MainWindow::buildWFAndDisplay_SWF(TMPacketToRead *packet, WFPacket *wfPacket, unsigned char num_page)
{
    unsigned int i = 0;
    static unsigned int nbSamples = 0;
    unsigned char *data;
    unsigned char pa_lfr_pkt_cnt;
    unsigned char pa_lfr_pkt_nr;
    unsigned int pa_lfr_swf_blk_nr = 0;
    double deltaT;

    switch(num_page)
    {
    case 0: // F0
        deltaT = 1. / 24576;
        break;
    case 1: // F1
        deltaT = 1. / 4096;
        break;
    case 2: // F2
        deltaT = 1. / 256;
        break;
    default:
        deltaT = 0;
        break;
    }
    pa_lfr_pkt_cnt = packet->Value[24];
    pa_lfr_pkt_nr = packet->Value[25];                                  // PA_LFR_PKT_NR
    pa_lfr_swf_blk_nr = packet->Value[32] * 256 + packet->Value[33];    // PA_LFR_SWF_BLK_NR
    data = &packet->Value[34]; // start of the first data block;

    if (pa_lfr_pkt_nr == 1) // the acquisition time of the first packet of the snapshot is taken as starting time
    {
        nbSamples = 0;
        wfPacket->coarseTime = ((packet->Value[26] & 0x7f) << 24)
                + (packet->Value[27] << 16)
                + (packet->Value[28] << 8)
                + packet->Value[29];
        wfPacket->fineTime = (packet->Value[30] << 8) + packet->Value[31];
    }

    for ( i=0; i<pa_lfr_swf_blk_nr; i++ ){
        wfPacket->wf_v[nbSamples + i]  = (short) ( (data[ (i * BLK_SIZE)     ] << 8) + (data[ (i*BLK_SIZE) + 1])  );
        wfPacket->wf_e1[nbSamples + i] = (short) ( (data[ (i * BLK_SIZE)  + 2] << 8) + (data[ (i*BLK_SIZE) + 3])  );
        wfPacket->wf_e2[nbSamples + i] = (short) ( (data[ (i * BLK_SIZE)  + 4] << 8) + (data[ (i*BLK_SIZE) + 5])  );
        wfPacket->wf_b1[nbSamples + i] = (short) ( (data[ (i * BLK_SIZE)  + 6] << 8) + (data[ (i*BLK_SIZE) + 7])  );
        wfPacket->wf_b2[nbSamples + i] = (short) ( (data[ (i * BLK_SIZE)  + 8] << 8) + (data[ (i*BLK_SIZE) + 9])  );
        wfPacket->wf_b3[nbSamples + i] = (short) ( (data[ (i * BLK_SIZE) + 10] << 8) + (data[ (i*BLK_SIZE) + 11]) );
    }

    nbSamples = nbSamples + pa_lfr_swf_blk_nr;

    if (pa_lfr_pkt_nr == pa_lfr_pkt_cnt)
    {
        this->UI->wfDisplay->displayOnPlot(wfPacket->wf_v,  num_page, 0, wfPacket->coarseTime, wfPacket->fineTime, deltaT, nbSamples);
        this->UI->wfDisplay->displayOnPlot(wfPacket->wf_e1, num_page, 1, wfPacket->coarseTime, wfPacket->fineTime, deltaT, nbSamples);
        this->UI->wfDisplay->displayOnPlot(wfPacket->wf_e2, num_page, 2, wfPacket->coarseTime, wfPacket->fineTime, deltaT, nbSamples);
        this->UI->wfDisplay->displayOnPlot(wfPacket->wf_b1, num_page, 3, wfPacket->coarseTime, wfPacket->fineTime, deltaT, nbSamples);
        this->UI->wfDisplay->displayOnPlot(wfPacket->wf_b2, num_page, 4, wfPacket->coarseTime, wfPacket->fineTime, deltaT, nbSamples);
        this->UI->wfDisplay->displayOnPlot(wfPacket->wf_b3, num_page, 5, wfPacket->coarseTime, wfPacket->fineTime, deltaT, nbSamples);
    }
}

void MainWindow::buildWFAndDisplay_SWF_alt(TMPacketToRead *packet, WFPacket *wfPacket, unsigned char num_page)
{
    unsigned int i = 0;
    unsigned int offset = 0;
    unsigned char *data;
    unsigned char pa_lfr_pkt_cnt;
    unsigned char pa_lfr_pkt_nr;
    unsigned int pa_lfr_swf_blk_nr = 0;
    double deltaT;
    bool flag_continue_analyze = true;

    switch(num_page)
    {
    case 0: // F0
        deltaT = 1. / 24576;
        break;
    case 1: // F1
        deltaT = 1. / 4096;
        break;
    case 2: // F2
        deltaT = 1. / 256;
        break;
    default:
        deltaT = 0;
        break;
    }

    pa_lfr_pkt_cnt = packet->Value[24];
    pa_lfr_pkt_nr = packet->Value[25];                                  // PA_LFR_PKT_NR
    pa_lfr_swf_blk_nr = packet->Value[32] * 256 + packet->Value[33];    // PA_LFR_SWF_BLK_NR
    data = &packet->Value[34]; // start of the first data block;

    //***************************
    // check parameters coherency
    if (flag_continue_analyze == true)  // PKT_CNT
    {
        if (pa_lfr_pkt_cnt != 7)
        {
            flag_continue_analyze = false;
            this->displayOnConsole("in buildWFAndDisplay_SWF *** pa_lfr_pkt_cnt = " + QString::number(pa_lfr_pkt_cnt));
        }
    }
    if (flag_continue_analyze == true)  // PKT_NR and BLK_NR
    {
        switch (pa_lfr_pkt_nr)
        {
        case 1:
        case 2:
        case 3:
        case 4:
        case 5:
        case 6:
            if (pa_lfr_swf_blk_nr != BLK_NR_304)
            {
                flag_continue_analyze = false;
                this->displayOnConsole("in buildWFAndDisplay_SWF *** pa_lfr_swf_blk_nr = " + QString::number(pa_lfr_swf_blk_nr));
            }
            break;
        case 7:
            if (pa_lfr_swf_blk_nr != BLK_NR_224)
            {
                flag_continue_analyze = false;
                this->displayOnConsole("in buildWFAndDisplay_SWF *** pa_lfr_swf_blk_nr = " + QString::number(pa_lfr_swf_blk_nr));
            }
            break;
        default:
            flag_continue_analyze = false;
            this->displayOnConsole("in buildWFAndDisplay_SWF *** pa_lfr_pkt_nr = " + QString::number(pa_lfr_pkt_nr));
            break;
        }
    }

    //*************
    // process data
    if (flag_continue_analyze == true)
    {
        if (pa_lfr_pkt_nr == 1) // the acquisition time of the first packet of the snapshot is taken as starting time
        {
            wfPacket->resetWaveforms(); // set all value to zero
            wfPacket->coarseTime = ((packet->Value[26] & 0x7f) << 24)
                    + (packet->Value[27] << 16)
                    + (packet->Value[28] << 8)
                    + packet->Value[29];
            wfPacket->fineTime = (packet->Value[30] << 8) + packet->Value[31];
        }

        for ( i=0; i<pa_lfr_swf_blk_nr; i++ ){
            offset = (pa_lfr_pkt_nr-1) * BLK_NR_304;
            wfPacket->wf_v [offset + i] = (short) ( (data[ (i * BLK_SIZE)     ] << 8) + (data[ (i*BLK_SIZE) + 1])  );
            wfPacket->wf_e1[offset + i] = (short) ( (data[ (i * BLK_SIZE)  + 2] << 8) + (data[ (i*BLK_SIZE) + 3])  );
            wfPacket->wf_e2[offset + i] = (short) ( (data[ (i * BLK_SIZE)  + 4] << 8) + (data[ (i*BLK_SIZE) + 5])  );
            wfPacket->wf_b1[offset + i] = (short) ( (data[ (i * BLK_SIZE)  + 6] << 8) + (data[ (i*BLK_SIZE) + 7])  );
            wfPacket->wf_b2[offset + i] = (short) ( (data[ (i * BLK_SIZE)  + 8] << 8) + (data[ (i*BLK_SIZE) + 9])  );
            wfPacket->wf_b3[offset + i] = (short) ( (data[ (i * BLK_SIZE) + 10] << 8) + (data[ (i*BLK_SIZE) + 11]) );
        }

        if (pa_lfr_pkt_nr == pa_lfr_pkt_cnt) // if this is the last packet of the snapshot, launch the display (and the record)
        {
            this->UI->wfDisplay->displayOnPlot(wfPacket->wf_v,  num_page, 0, wfPacket->coarseTime, wfPacket->fineTime, deltaT, 2048);
            this->UI->wfDisplay->displayOnPlot(wfPacket->wf_e1, num_page, 1, wfPacket->coarseTime, wfPacket->fineTime, deltaT, 2048);
            this->UI->wfDisplay->displayOnPlot(wfPacket->wf_e2, num_page, 2, wfPacket->coarseTime, wfPacket->fineTime, deltaT, 2048);
            this->UI->wfDisplay->displayOnPlot(wfPacket->wf_b1, num_page, 3, wfPacket->coarseTime, wfPacket->fineTime, deltaT, 2048);
            this->UI->wfDisplay->displayOnPlot(wfPacket->wf_b2, num_page, 4, wfPacket->coarseTime, wfPacket->fineTime, deltaT, 2048);
            this->UI->wfDisplay->displayOnPlot(wfPacket->wf_b3, num_page, 5, wfPacket->coarseTime, wfPacket->fineTime, deltaT, 2048);
        }
    }
}

void MainWindow::buildWFAndDisplay_CWF_LONG_F3(TMPacketToRead *packet, WFPacket *wfPacket)
{
    unsigned int i = 0;
    unsigned int j = 0;
    unsigned char *data;
    static unsigned char pkt_nr = 1;
    unsigned int blk_nr = 0;
    static unsigned int coarseTime;
    static unsigned int fineTime;
    double deltaT;
    unsigned int nbData;

    deltaT = 1 / 16.;

    if (pkt_nr == 1) // the acquisition time of the first packet of the snapshot is taken as starting time
    {
        coarseTime = ((packet->Value[24] & 0x7f) << 24)
                + (packet->Value[25] << 16)
                + (packet->Value[26] << 8)
                + packet->Value[27];
        fineTime = (packet->Value[28] << 8) + packet->Value[29];
    }
    blk_nr = packet->Value[30] * 256 + packet->Value[31];       // PA_LFR_CWFL3_BLK_NR
    data = &packet->Value[32]; // start of the first data block;

    j = (pkt_nr-1) * blk_nr;
    for ( i=0; i<blk_nr; i++ ){
        wfPacket->wf_v[j + i]  = (short) ( (data[ (i * BLK_SIZE)     ] << 8) + (data[ (i*BLK_SIZE) + 1])  );
        wfPacket->wf_e1[j + i] = (short) ( (data[ (i * BLK_SIZE)  + 2] << 8) + (data[ (i*BLK_SIZE) + 3])  );
        wfPacket->wf_e2[j + i] = (short) ( (data[ (i * BLK_SIZE)  + 4] << 8) + (data[ (i*BLK_SIZE) + 5])  );
        wfPacket->wf_b1[j + i] = (short) ( (data[ (i * BLK_SIZE)  + 6] << 8) + (data[ (i*BLK_SIZE) + 7])  );
        wfPacket->wf_b2[j + i] = (short) ( (data[ (i * BLK_SIZE)  + 8] << 8) + (data[ (i*BLK_SIZE) + 9])  );
        wfPacket->wf_b3[j + i] = (short) ( (data[ (i * BLK_SIZE) + 10] << 8) + (data[ (i*BLK_SIZE) + 11]) );
    }
    pkt_nr = pkt_nr + 1;
    if (pkt_nr == (NB_PACKETS_PER_GROUP_OF_CWF+1))
    {
        nbData = j + blk_nr;
        this->UI->wfPage_CWF_F3->displayOnPlot(wfPacket->wf_v, 0, coarseTime, fineTime, deltaT, nbData);
        this->UI->wfPage_CWF_F3->displayOnPlot(wfPacket->wf_e1, 1, coarseTime, fineTime, deltaT, nbData);
        this->UI->wfPage_CWF_F3->displayOnPlot(wfPacket->wf_e2, 2, coarseTime, fineTime, deltaT, nbData);
        this->UI->wfPage_CWF_F3->displayOnPlot(wfPacket->wf_b1, 3, coarseTime, fineTime, deltaT, nbData);
        this->UI->wfPage_CWF_F3->displayOnPlot(wfPacket->wf_b2, 4, coarseTime, fineTime, deltaT, nbData);
        this->UI->wfPage_CWF_F3->displayOnPlot(wfPacket->wf_b3, 5, coarseTime, fineTime, deltaT, nbData);
        pkt_nr = 1;
    }
}

void MainWindow::buildWFAndDisplay_CWF_F3_light(TMPacketToRead *packet, WFPacket *wfPacket)
{
    unsigned int i = 0;
    unsigned int j = 0;
    unsigned char *data;
    static unsigned char pkt_nr = 1;
    unsigned int blk_nr = 0;
    static unsigned int coarseTime;
    static unsigned int fineTime;
    double deltaT;
    unsigned int nbData;

    deltaT = 1 / 16.;

    blk_nr = packet->Value[30] * 256 + packet->Value[31];       // PA_LFR_CWF3_BLK_NR
    data = &packet->Value[32]; // start of the first data block;

    if (pkt_nr == 1)
    {
        coarseTime = ((packet->Value[24] & 0x7f) << 24)
                + (packet->Value[25] << 16)
                + (packet->Value[26] << 8)
                + packet->Value[27];
        fineTime = (packet->Value[28] << 8) + packet->Value[29];
        this->displayOnConsole( "[0] = " + QString::number( data[0] , 16)
                + ", [1] = " + QString::number( data[1] , 16)
                + ", [2] = " + QString::number( data[2] , 16)
                + ", [3] = " + QString::number( data[3] , 16)
                + ", [4] = " + QString::number( data[4] , 16)
                + ", [5] = " + QString::number( data[5] , 16));
    }

    j = (pkt_nr-1) * blk_nr;
    for ( i=0; i<blk_nr; i++ ){
        wfPacket->wf_v[j + i]  = (short) ( (data[ (i * BLK_SIZE_CWF3_LIGHT)    ] << 8) + (data[ (i*BLK_SIZE_CWF3_LIGHT) + 1])  );
        wfPacket->wf_e1[j + i] = (short) ( (data[ (i * BLK_SIZE_CWF3_LIGHT) + 2] << 8) + (data[ (i*BLK_SIZE_CWF3_LIGHT) + 3])  );
        wfPacket->wf_e2[j + i] = (short) ( (data[ (i * BLK_SIZE_CWF3_LIGHT) + 4] << 8) + (data[ (i*BLK_SIZE_CWF3_LIGHT) + 5])  );
        wfPacket->wf_b1[j + i] = 0;
        wfPacket->wf_b2[j + i] = 0;
        wfPacket->wf_b3[j + i] = 0;
    }
    pkt_nr = pkt_nr + 1;
    if (pkt_nr == (NB_PACKETS_PER_GROUP_OF_CWF_LIGHT+1))
    {
        nbData = j + blk_nr;
        this->UI->wfPage_CWF_F3->displayOnPlot(wfPacket->wf_v, 0, coarseTime, fineTime, deltaT, nbData);
        this->UI->wfPage_CWF_F3->displayOnPlot(wfPacket->wf_e1, 1, coarseTime, fineTime, deltaT, nbData);
        this->UI->wfPage_CWF_F3->displayOnPlot(wfPacket->wf_e2, 2, coarseTime, fineTime, deltaT, nbData);
        this->UI->wfPage_CWF_F3->displayOnPlot(wfPacket->wf_b1, 3, coarseTime, fineTime, deltaT, nbData);
        this->UI->wfPage_CWF_F3->displayOnPlot(wfPacket->wf_b2, 4, coarseTime, fineTime, deltaT, nbData);
        this->UI->wfPage_CWF_F3->displayOnPlot(wfPacket->wf_b3, 5, coarseTime, fineTime, deltaT, nbData);
        pkt_nr = 1;
    }
}

void MainWindow::buildWFAndDisplay_CWF_F1(TMPacketToRead *packet, WFPacket *wfPacket)
{
    unsigned int i = 0;
    unsigned int j = 0;
    unsigned char *data;
    static unsigned char pkt_nr = 1;
    unsigned int blk_nr = 0;
    static unsigned int coarseTime;
    static unsigned int fineTime;
    double deltaT;
    unsigned int nbData;

    deltaT = 1. / 4096.;

    if (pkt_nr == 1)
    {
        coarseTime = ((packet->Value[24] & 0x7f) << 24)
                + (packet->Value[25] << 16)
                + (packet->Value[26] << 8)
                + packet->Value[27];
        fineTime = (packet->Value[28] << 8) + packet->Value[29];
    }
    blk_nr = packet->Value[30] * 256 + packet->Value[31];       // PA_LFR_CWF_BLK_NR
    data = &packet->Value[32]; // start of the first data block;

    j = (pkt_nr-1) * blk_nr;
    for ( i=0; i<blk_nr; i++ ){
        wfPacket->wf_v[j + i]  = (short) ( (data[ (i * BLK_SIZE)     ] << 8) + (data[ (i*BLK_SIZE) + 1])  );
        wfPacket->wf_e1[j + i] = (short) ( (data[ (i * BLK_SIZE)  + 2] << 8) + (data[ (i*BLK_SIZE) + 3])  );
        wfPacket->wf_e2[j + i] = (short) ( (data[ (i * BLK_SIZE)  + 4] << 8) + (data[ (i*BLK_SIZE) + 5])  );
        wfPacket->wf_b1[j + i] = (short) ( (data[ (i * BLK_SIZE)  + 6] << 8) + (data[ (i*BLK_SIZE) + 7])  );
        wfPacket->wf_b2[j + i] = (short) ( (data[ (i * BLK_SIZE)  + 8] << 8) + (data[ (i*BLK_SIZE) + 9])  );
        wfPacket->wf_b3[j + i] = (short) ( (data[ (i * BLK_SIZE) + 10] << 8) + (data[ (i*BLK_SIZE) + 11]) );
    }
    pkt_nr = pkt_nr + 1;
    if (pkt_nr == (NB_PACKETS_PER_GROUP_OF_CWF+1))
    {
        nbData = j + blk_nr;
        this->UI->wfPage_CWF_F1->displayOnPlot(wfPacket->wf_v,  0, coarseTime, fineTime, deltaT, nbData);
        this->UI->wfPage_CWF_F1->displayOnPlot(wfPacket->wf_e1, 1, coarseTime, fineTime, deltaT, nbData);
        this->UI->wfPage_CWF_F1->displayOnPlot(wfPacket->wf_e2, 2, coarseTime, fineTime, deltaT, nbData);
        this->UI->wfPage_CWF_F1->displayOnPlot(wfPacket->wf_b1, 3, coarseTime, fineTime, deltaT, nbData);
        this->UI->wfPage_CWF_F1->displayOnPlot(wfPacket->wf_b2, 4, coarseTime, fineTime, deltaT, nbData);
        this->UI->wfPage_CWF_F1->displayOnPlot(wfPacket->wf_b3, 5, coarseTime, fineTime, deltaT, nbData);
//        this->UI->spectrum_V->computeFFTAndDisplay( wfPacket->wf_b1, nbData );
        this->UI->allSpectra->spectrum[0]->computeFFTAndDisplay( wfPacket->wf_v, nbData );
        this->UI->allSpectra->spectrum[1]->computeFFTAndDisplay( wfPacket->wf_e1, nbData );
        this->UI->allSpectra->spectrum[2]->computeFFTAndDisplay( wfPacket->wf_e2, nbData );
        this->UI->allSpectra->spectrum[3]->computeFFTAndDisplay( wfPacket->wf_b1, nbData );
        this->UI->allSpectra->spectrum[4]->computeFFTAndDisplay( wfPacket->wf_b2, nbData );
        this->UI->allSpectra->spectrum[5]->computeFFTAndDisplay( wfPacket->wf_b3, nbData );
        pkt_nr = 1;
    }
}

void MainWindow::buildWFAndDisplay_CWF_F2(TMPacketToRead *packet, WFPacket *wfPacket)
{
    unsigned int i = 0;
    unsigned int j = 0;
    unsigned char *data;
    static unsigned char pkt_nr = 1;
    unsigned int blk_nr = 0;
    static unsigned int coarseTime;
    static unsigned int fineTime;
    double deltaT;
    unsigned int nbData;

    deltaT = 1. / 256.;

    if (pkt_nr == 1) // the acquisition time of the first packet of the snapshot is taken as starting time
    {
        coarseTime = ((packet->Value[24] & 0x7f) << 24)
                + (packet->Value[25] << 16)
                + (packet->Value[26] << 8)
                + packet->Value[27];
        fineTime = (packet->Value[28] << 8) + packet->Value[29];
    }
    blk_nr = packet->Value[30] * 256 + packet->Value[31];       // PA_LFR_CWF3_BLK_NR
    data = &packet->Value[32]; // start of the first data block;

    j = (pkt_nr-1) * blk_nr;
    for ( i=0; i<blk_nr; i++ ){
        wfPacket->wf_v[j + i]  = (short) ( (data[ (i * BLK_SIZE)     ] << 8) + (data[ (i*BLK_SIZE) + 1])  );
        wfPacket->wf_e1[j + i] = (short) ( (data[ (i * BLK_SIZE)  + 2] << 8) + (data[ (i*BLK_SIZE) + 3])  );
        wfPacket->wf_e2[j + i] = (short) ( (data[ (i * BLK_SIZE)  + 4] << 8) + (data[ (i*BLK_SIZE) + 5])  );
        wfPacket->wf_b1[j + i] = (short) ( (data[ (i * BLK_SIZE)  + 6] << 8) + (data[ (i*BLK_SIZE) + 7])  );
        wfPacket->wf_b2[j + i] = (short) ( (data[ (i * BLK_SIZE)  + 8] << 8) + (data[ (i*BLK_SIZE) + 9])  );
        wfPacket->wf_b3[j + i] = (short) ( (data[ (i * BLK_SIZE) + 10] << 8) + (data[ (i*BLK_SIZE) + 11]) );
    }
    pkt_nr = pkt_nr + 1;
    if (pkt_nr == (NB_PACKETS_PER_GROUP_OF_CWF+1))
    {
        nbData = j + blk_nr;
        this->UI->wfPage_CWF_F2->displayOnPlot(wfPacket->wf_v,  0, coarseTime, fineTime, deltaT, nbData);
        this->UI->wfPage_CWF_F2->displayOnPlot(wfPacket->wf_e1, 1, coarseTime, fineTime, deltaT, nbData);
        this->UI->wfPage_CWF_F2->displayOnPlot(wfPacket->wf_e2, 2, coarseTime, fineTime, deltaT, nbData);
        this->UI->wfPage_CWF_F2->displayOnPlot(wfPacket->wf_b1, 3, coarseTime, fineTime, deltaT, nbData);
        this->UI->wfPage_CWF_F2->displayOnPlot(wfPacket->wf_b2, 4, coarseTime, fineTime, deltaT, nbData);
        this->UI->wfPage_CWF_F2->displayOnPlot(wfPacket->wf_b3, 5, coarseTime, fineTime, deltaT, nbData);
        pkt_nr = 1;
    }
}

float MainWindow::rebuildFloat( unsigned char *data )
{
    float value;
    unsigned char *valueCharPtr;

    valueCharPtr = (unsigned char*) &value;

    valueCharPtr[3] = data[0];
    valueCharPtr[2] = data[1];
    valueCharPtr[1] = data[2];
    valueCharPtr[0] = data[3];

    return value;
}

void MainWindow::build_ASM_AndDisplay_NORM(TMPacketToRead *packet, SpectraPacket *spectraPacket,
                                           double deltaF, unsigned int indiceStart,
                                           ASMPage* asmPage )
{
    unsigned int i;
    unsigned int j;
    unsigned char *data;
    unsigned char cntASM;
    unsigned char nrASM;
    unsigned int blkNR;
    unsigned int nbData;
    unsigned char sid;
    unsigned char blkNRStep;

    sid = packet->Value[20];
    cntASM = packet->Value[24];
    nrASM = packet->Value[25];
    blkNR = packet->Value[32] * 256 + packet->Value[33];

    data = &packet->Value[34]; // start of the first data block;

    switch (sid) {
    case SID_NORMAL_ASM_F0:
        blkNRStep = 32; // 32 + 32 + 24 = 88
        break;
    case SID_NORMAL_ASM_F1:
        blkNRStep = 36; // 36 + 36 + 32 = 104
        break;
    case SID_NORMAL_ASM_F2:
        blkNRStep = 32; // 32 + 32 + 32 = 96
        break;
    default:
        emit displayOnConsole("in build_ASM_AndDisplay_NORM *** wrong sid");
        blkNRStep = 0;
        break;
    }

    j = (nrASM-1) * blkNRStep;

    for ( i=0; i<blkNR; i++ ){
        //***
        // B1
        spectraPacket->b1b1[    j + i ] = rebuildFloat( &data[ (i*100) + 4*0 ]  );
        spectraPacket->b1b2_re[ j + i ] = rebuildFloat( &data[ (i*100) + 4*1 ]  );
        spectraPacket->b1b2_im[ j + i ] = rebuildFloat( &data[ (i*100) + 4*2 ]  );
        spectraPacket->b1b3_re[ j + i ] = rebuildFloat( &data[ (i*100) + 4*3 ]  );
        spectraPacket->b1b3_im[ j + i ] = rebuildFloat( &data[ (i*100) + 4*4 ]  );
        spectraPacket->b1e1_re[ j + i ] = rebuildFloat( &data[ (i*100) + 4*5 ]  );
        spectraPacket->b1e1_im[ j + i ] = rebuildFloat( &data[ (i*100) + 4*6 ]  );
        spectraPacket->b1e2_re[ j + i ] = rebuildFloat( &data[ (i*100) + 4*7 ]  );
        spectraPacket->b1e2_im[ j + i ] = rebuildFloat( &data[ (i*100) + 4*8 ]  );
        //***
        // B2
        spectraPacket->b2b2[    j + i ] = rebuildFloat( &data[ (i*100) + 4*9 ]  );
        spectraPacket->b2b3_re[ j + i ] = rebuildFloat( &data[ (i*100) + 4*10 ]  );
        spectraPacket->b2b3_im[ j + i ] = rebuildFloat( &data[ (i*100) + 4*11 ]  );
        spectraPacket->b2e1_re[ j + i ] = rebuildFloat( &data[ (i*100) + 4*12 ]  );
        spectraPacket->b2e1_im[ j + i ] = rebuildFloat( &data[ (i*100) + 4*13 ]  );
        spectraPacket->b2e2_re[ j + i ] = rebuildFloat( &data[ (i*100) + 4*14 ]  );
        spectraPacket->b2e2_im[ j + i ] = rebuildFloat( &data[ (i*100) + 4*15 ]  );
        //***
        // B3
        spectraPacket->b3b3[    j + i ] = rebuildFloat( &data[ (i*100) + 4*16 ]  );
        spectraPacket->b3e1_re[ j + i ] = rebuildFloat( &data[ (i*100) + 4*17 ]  );
        spectraPacket->b3e1_im[ j + i ] = rebuildFloat( &data[ (i*100) + 4*18 ]  );
        spectraPacket->b3e2_re[ j + i ] = rebuildFloat( &data[ (i*100) + 4*19 ]  );
        spectraPacket->b3e2_im[ j + i ] = rebuildFloat( &data[ (i*100) + 4*20 ]  );
        //***
        // E1
        spectraPacket->e1e1[    j + i ] = rebuildFloat( &data[ (i*100) + 4*21 ]  );
        spectraPacket->e1e2_re[ j + i ] = rebuildFloat( &data[ (i*100) + 4*22 ]  );
        spectraPacket->e1e2_im[ j + i ] = rebuildFloat( &data[ (i*100) + 4*23 ]  );
        //***
        // E2
        spectraPacket->e2e2[    j + i ] = rebuildFloat( &data[ (i*100) + 4*24 ]  );
    }

    nbData = j + blkNR;
    if (nrASM == cntASM)
    {
        asmPage->storeSpectraPacket( spectraPacket, indiceStart, deltaF, nbData );

        if (this->UI->checkbox_displayASMInDB->isChecked() )
        {
            dBConversion( spectraPacket->b1b1, nbData );
            dBConversion( spectraPacket->b2b2, nbData );
            dBConversion( spectraPacket->b3b3, nbData );
            dBConversion( spectraPacket->e1e1, nbData );
            dBConversion( spectraPacket->e2e2, nbData );
        }
        asmPage->displayOnPlot( spectraPacket->b1b1, 0, deltaF, indiceStart, nbData );
        asmPage->displayOnPlot( spectraPacket->b2b2, 1, deltaF, indiceStart, nbData );
        asmPage->displayOnPlot( spectraPacket->b3b3, 2, deltaF, indiceStart, nbData );
        asmPage->displayOnPlot( spectraPacket->e1e1, 3, deltaF, indiceStart, nbData );
        asmPage->displayOnPlot( spectraPacket->e2e2, 4, deltaF, indiceStart, nbData );
    }
}

void MainWindow::dBConversion( float *dataBuffer, unsigned int nbData )
{
    unsigned int k;

    for( k=0; k<nbData; k++)
    {
        dataBuffer[k] = 10 * log10( dataBuffer[ k ] );
    }
}

void MainWindow::extract_v_e1_e2_temperatures( TMPacketToRead *packet )
{
    short v;
    short e1;
    short e2;
    short temp_scm;
    short temp_pcb;
    short temp_fpga;

    unsigned int coarseTime;
    unsigned int fineTime;

    coarseTime = ((packet->Value[14] & 0x7f) << 24)
            + (packet->Value[15] << 16)
            + (packet->Value[16] << 8)
            + packet->Value[17];
    fineTime = (packet->Value[18] << 8) + packet->Value[19];

    temp_scm = (short) (packet->Value[104] << 8) + packet->Value[105];
    temp_pcb = (short) (packet->Value[106] << 8) + packet->Value[107];
    temp_fpga = (short) (packet->Value[108] << 8) + packet->Value[109];

    v = (short) (packet->Value[110] << 8) + packet->Value[111];
    e1 = (short) (packet->Value[112] << 8) + packet->Value[113];
    e2 = (short) (packet->Value[114] << 8) + packet->Value[115];

    this->UI->storeHK->buildDataAsQByteArray(temp_scm, temp_pcb, temp_fpga, v, e1, e2, coarseTime, fineTime);
}

void MainWindow::resetStatistics()
{
    totalOfBytes = 0;
    totalOfPackets = 0;
    this->UI->totalOfBytesHasChanged(totalOfBytes);
    this->UI->totalOfPacketsHasChanged(totalOfPackets);
}

//******************
// packet processing

void MainWindow::processPacketStore()
{
    TMPacketToRead *packet;
    while(!generalCCSDSPacketStore.isEmpty())
    {
        packet = generalCCSDSPacketStore.takeFirst();
        processIncomingPacket(packet);
        delete(packet);
    }
}

void MainWindow::processIncomingPacket(TMPacketToRead *packet)
{
    totalOfBytes = totalOfBytes + packet->size;
    totalOfPackets = totalOfPackets + 1;
    this->UI->totalOfBytesHasChanged(totalOfBytes);
    this->UI->totalOfPacketsHasChanged(totalOfPackets);
    preProcessPacket(packet);
}

void MainWindow::preProcessPacket(TMPacketToRead *packet)
{
    unsigned char pid = 0;
    unsigned char cat = 0;
    unsigned char typ = 0;
    unsigned char sub = 0;
    unsigned int sid = 0;
    unsigned int length = 0;
    unsigned int coarse_t = 0;
    unsigned int fine_t = 0;

    bool packetCompliant = true;

    //***************************************
    // get the length parameter
    length      = this->UI->tmStatistics->getLENGTH( packet );

    // compare length in the packet with the size of the packet
    if ( (length + 1 + 10) != (packet->size))
    {
        displayOnConsole("Packet dropped: reception of " + QString::number(packet->size)
                         + " bytes instead of " + QString::number(length + 1 + 10) );
        packetCompliant = false;
    }

    if (packetCompliant == true)
    {
        //***************************************
        // get the other parameters of the packet
        pid         = this->UI->tmStatistics->getPID( packet );
        cat         = this->UI->tmStatistics->getCAT( packet );
        typ         = this->UI->tmStatistics->getTYPE( packet );
        sub         = this->UI->tmStatistics->getSUBTYPE( packet );
        coarse_t    = this->UI->tmStatistics->getCoarseTime( packet );
        fine_t      = this->UI->tmStatistics->getFineTime( packet );
        sid         = this->UI->tmStatistics->getSID( packet, pid, cat, typ, sub );

        packetCompliant = this->UI->tmStatistics->updateStatistics(pid, cat, typ, sub, sid, length, coarse_t, fine_t);
    }

    if (packetCompliant == true)
    {
        //***************************************************
        // if the packet is an HK packet, display its content
        if ( (pid == TM_PACKET_PID_DEFAULT) & (cat == TM_PACKET_CAT_HK)
             & (typ == TM_TYPE_HK) & (sub == TM_SUBTYPE_HK) )
        {
            this->UI->hkDisplay->displayPacket( packet );
            extract_v_e1_e2_temperatures( packet );
            this->UI->displayLFRConfig->processCommonParameters( packet->Value[117] );
        }

        //**************************************************************
        // if the packet is a parameter dump packet, display its content
        if ( (pid == TM_PACKET_PID_DEFAULT) & (cat == TM_PACKET_CAT_DUMP)
             & (typ == TM_TYPE_PARAMETER_DUMP) & (sub == TM_SUBTYPE_PARAMETER_DUMP) )
        {
            this->UI->parameterDump->updateParameterDump(packet);
            this->UI->displayLFRConfig->processCommonParameters( packet->Value[22] );
        }

        //*************************************************************
        // if the packet is a waveform or a spectral matrix, display it
        if ( (typ == TM_TYPE_LFR_SCIENCE) & (sub == TM_SUBTYPE_LFR_SCIENCE_6) )
        {
            switch (sid){
            case SID_NORMAL_SWF_F0:
                buildWFAndDisplay_SWF_alt(packet, wfPacketNormal[0], 0);
                break;
            case SID_NORMAL_SWF_F1:
                buildWFAndDisplay_SWF_alt(packet, wfPacketNormal[1], 1);
                break;
            case SID_NORMAL_SWF_F2:
                buildWFAndDisplay_SWF_alt(packet, wfPacketNormal[2], 2);
                break;
            case SID_NORMAL_CWF_F3:
                buildWFAndDisplay_CWF_F3_light(packet, wfPacketCWF_F3);
                break;
            case SID_BURST_CWF_F2:
                buildWFAndDisplay_CWF_F2(packet, wfPacketBurst);
                break;
            case SID_SBM1_CWF_F1:
                buildWFAndDisplay_CWF_F1(packet, wfPacketSBM1);
                break;
            case SID_SBM2_CWF_F2:
                buildWFAndDisplay_CWF_F2(packet, wfPacketSBM2);
                break;
            case SID_NORMAL_ASM_F0: // +1  is due to the fact that the VHDL code shifts the frequencies in the spectral matrices
                build_ASM_AndDisplay_NORM(packet, spectraPacketNormalSpectrumF0, 24576./256., ASM_F0_INDICE_START + 1, this->UI->asmPage_F0 );
                break;
            case SID_NORMAL_ASM_F1: // +1  is due to the fact that the VHDL code shifts the frequencies in the spectral matrices
                build_ASM_AndDisplay_NORM(packet, spectraPacketNormalSpectrumF1, 4096./256., ASM_F1_INDICE_START + 1, this->UI->asmPage_F1 );
                break;
            case SID_NORMAL_CWF_LONG_F3:
                buildWFAndDisplay_CWF_LONG_F3(packet, wfPacketCWF_F3);
                break;
            }
        }

        if ( (typ == TM_TYPE_LFR_SCIENCE) & (sub == TM_SUBTYPE_LFR_SCIENCE_3) )
        {
            switch (sid){
            case SID_NORMAL_ASM_F2: // +1  is due to the fact that the VHDL code shifts the frequencies in the spectral matrices
                build_ASM_AndDisplay_NORM(packet, spectraPacketNormalSpectrumF2, 256./256., ASM_F2_INDICE_START + 1, this->UI->asmPage_F2 );
                break;
            }
        }
    }
}

//******************
// general functions

void MainWindow::readSettings()
{
    QSettings settings("lpp", "lfrsgse");
    echoBridgeIP0 = (unsigned char) settings.value("echoBridgeIP0", 127).toInt();
    echoBridgeIP1 = (unsigned char) settings.value("echoBridgeIP1", 0).toInt();
    echoBridgeIP2 = (unsigned char) settings.value("echoBridgeIP2", 0).toInt();
    echoBridgeIP3 = (unsigned char) settings.value("echoBridgeIP3", 1).toInt();
}

void MainWindow::writeSettings()
{
    QSettings settings("lpp", "lfrsgse");
    settings.setValue("echoBridgeIP0", this->UI->serverTMEchoDialogBox->get_addressPart1());
    settings.setValue("echoBridgeIP1", this->UI->serverTMEchoDialogBox->get_addressPart2());
    settings.setValue("echoBridgeIP2", this->UI->serverTMEchoDialogBox->get_addressPart3());
    settings.setValue("echoBridgeIP3", this->UI->serverTMEchoDialogBox->get_addressPart4());
}

void MainWindow::closeEvent(QCloseEvent *)
{
    writeSettings();
}
