#include "mainwindowui.h"

MainWindowUI::MainWindowUI(QWidget *parent) :
    QWidget(parent)
{
    buildWelcomePage();

    spwTabWidget = new QTabWidget();
    spwTabWidgetPage0 = new QWidget();
    spwTabWidgetPage1 = new QWidget();
    spwTabWidgetPage2 = new QWidget();
    spwTabWidgetPage3 = new QWidget();

    tmStatistics = new TMStatistics;

    wfDisplay   = new WFDisplay(0, 2048, 2048, 15000);
    hkDisplay = new HKDisplay();
    recordPage = new RecordPage();
    wfPage_CWF_F1  = new WFPage(0, 8 * BLK_NR_CWF, 8 * BLK_NR_CWF, 15000);
    wfPage_CWF_F2  = new WFPage(0, 8 * BLK_NR_CWF, 8 * BLK_NR_CWF, 15000);
    wfPage_CWF_F3  = new WFPage(0, 8 * BLK_NR_CWF, 8 * BLK_NR_CWF, 15000);
    asmPage_F0 = new ASMPage(0, NB_BINS_PER_ASM_F0, NB_BINS_PER_ASM_F0, 15000);
    asmPage_F1 = new ASMPage(0, NB_BINS_PER_ASM_F1, NB_BINS_PER_ASM_F1, 15000);
    asmPage_F2 = new ASMPage(0, NB_BINS_PER_ASM_F2, NB_BINS_PER_ASM_F2, 15000);
    storeHK = new StoreHK();
    allSpectra = new spectra( this, 6, 100, 2688/2+1, this->getScaleFactor() );

    allSpectra->setTabTitle( 0, "V");
    allSpectra->setTabTitle( 1, "E1");
    allSpectra->setTabTitle( 2, "E2");
    allSpectra->setTabTitle( 3, "B1");
    allSpectra->setTabTitle( 4, "B2");
    allSpectra->setTabTitle( 5, "B3");

    wfPage_CWF_F1->pageTitle = "SBM1_CWF_F1";
    wfPage_CWF_F2->pageTitle = "SBM2_CWF_F2";
    wfPage_CWF_F3->pageTitle = "NORM_CWF_F3";
    asmPage_F0->pageTitle = "ASM_F0";
    asmPage_F1->pageTitle = "ASM_F1";
    asmPage_F2->pageTitle = "ASM_F2";
    storeHK->hkName = "spacecraft_potential";

    layout_overallLayout = new QVBoxLayout();

    parameterDump = new ParameterDump();

    spwTabWidget->addTab(spwTabWidgetPage0, tr("Connection"));
    spwTabWidget->addTab(spwTabWidgetPage1, tr("TM Statistics"));
    spwTabWidget->addTab(spwTabWidgetPage3, tr("HK"));
    spwTabWidget->addTab(parameterDump, tr("DUMP"));
    spwTabWidget->addTab(recordPage, tr("REC"));
    spwTabWidget->addTab(spwTabWidgetPage2, tr("SWF"));
    spwTabWidget->addTab( wfPage_CWF_F1, tr("CWF_F1"));
    spwTabWidget->addTab( allSpectra, tr("FFT"));
    spwTabWidget->addTab( wfPage_CWF_F2, tr("CWF_F2"));
    spwTabWidget->addTab( wfPage_CWF_F3, tr("CWF_F3"));
    spwTabWidget->addTab( asmPage_F0, tr("ASM_F0"));
    spwTabWidget->addTab( asmPage_F1, tr("ASM_F1"));
    spwTabWidget->addTab( asmPage_F2, tr("ASM_F2"));

    spwTabWidgetPage0->setLayout(layout_welcomePage);
    spwTabWidgetPage1->setLayout(tmStatistics->layout());
    spwTabWidgetPage2->setLayout(wfDisplay->layout());
    spwTabWidgetPage3->setLayout(hkDisplay->layout());

    layout_overallLayout->addWidget(spwTabWidget);

    this->setLayout(layout_overallLayout);

    connect(this->button_clearConsole, SIGNAL(clicked()), this, SLOT(clearConsole()));

    // SWF_F0
    connect(this->recordPage, SIGNAL(signal_store_swf_f0()),
            this->wfDisplay->page_f0, SLOT(storeWfrm()));
    connect(this->recordPage, SIGNAL(signal_defaultStorageDirectoryUpdated(QString)),
            this->wfDisplay->page_f0, SLOT(setDefaultStorageDirectory(QString)));
    // SWF_F1
    connect(this->recordPage, SIGNAL(signal_store_swf_f1()),
            this->wfDisplay->page_f1, SLOT(storeWfrm()));
    connect(this->recordPage, SIGNAL(signal_defaultStorageDirectoryUpdated(QString)),
            this->wfDisplay->page_f1, SLOT(setDefaultStorageDirectory(QString)));
    // SWF_F2
    connect(this->recordPage, SIGNAL(signal_store_swf_f2()),
            this->wfDisplay->page_f2, SLOT(storeWfrm()));
    connect(this->recordPage, SIGNAL(signal_defaultStorageDirectoryUpdated(QString)),
            this->wfDisplay->page_f2, SLOT(setDefaultStorageDirectory(QString)));
    // CWF_F1
    connect(this->recordPage, SIGNAL(signal_store_cwf_f1()),
            this->wfPage_CWF_F1, SLOT(storeWfrm()));
    connect(this->recordPage, SIGNAL(signal_defaultStorageDirectoryUpdated(QString)),
            this->wfPage_CWF_F1, SLOT(setDefaultStorageDirectory(QString)));
    // CWF_F2
    connect(this->recordPage, SIGNAL(signal_store_cwf_f2()),
            this->wfPage_CWF_F2, SLOT(storeWfrm()));
    connect(this->recordPage, SIGNAL(signal_defaultStorageDirectoryUpdated(QString)),
            this->wfPage_CWF_F2, SLOT(setDefaultStorageDirectory(QString)));
    // CWF_F3
    connect(this->recordPage, SIGNAL(signal_store_cwf_f3()),
            this->wfPage_CWF_F3, SLOT(storeWfrm()));
    connect(this->recordPage, SIGNAL(signal_defaultStorageDirectoryUpdated(QString)),
            this->wfPage_CWF_F3, SLOT(setDefaultStorageDirectory(QString)));
    // ASM_F0
    connect(this->recordPage, SIGNAL(signal_store_asm_f0()),
            this->asmPage_F0, SLOT(storeWfrm()));
    connect(this->recordPage, SIGNAL(signal_defaultStorageDirectoryUpdated(QString)),
            this->asmPage_F0, SLOT(setDefaultStorageDirectory(QString)));
    // ASM_F1
    connect(this->recordPage, SIGNAL(signal_store_asm_f1()),
            this->asmPage_F1, SLOT(storeWfrm()));
    connect(this->recordPage, SIGNAL(signal_defaultStorageDirectoryUpdated(QString)),
            this->asmPage_F1, SLOT(setDefaultStorageDirectory(QString)));
    // ASM_F2
    connect(this->recordPage, SIGNAL(signal_store_asm_f2()),
            this->asmPage_F2, SLOT(storeWfrm()));
    connect(this->recordPage, SIGNAL(signal_defaultStorageDirectoryUpdated(QString)),
            this->asmPage_F2, SLOT(setDefaultStorageDirectory(QString)));
    // HK_SPACECRAFT_POTENTIAL
    connect(this->recordPage, SIGNAL(signal_store_hk_spacecraft_potential()),
            this->storeHK, SLOT(storeData()));
    connect(this->recordPage, SIGNAL(signal_defaultStorageDirectoryUpdated(QString)),
            this->storeHK, SLOT(setDefaultStorageDirectory(QString)));
    // SCALE FACTOR
    connect(this->spinBox_scaleFactor, SIGNAL(valueChanged(double)),
            this->allSpectra, SLOT(updateScaleFactor(double)));
    connect(this->spinBox_scaleFactor, SIGNAL(valueChanged(double)),
            this->wfDisplay, SLOT(updateScaleFactor(double)));
    connect(this->spinBox_scaleFactor, SIGNAL(valueChanged(double)),
            this->wfPage_CWF_F1, SLOT(updateScaleFactor(double)));
    connect(this->spinBox_scaleFactor, SIGNAL(valueChanged(double)),
            this->wfPage_CWF_F2, SLOT(updateScaleFactor(double)));
    connect(this->spinBox_scaleFactor, SIGNAL(valueChanged(double)),
            this->wfPage_CWF_F3, SLOT(updateScaleFactor(double)));
    // COLOR SCALE FACTOR
    connect(this->spinBox_maxDataColorScale, SIGNAL(valueChanged(double)),
            this->allSpectra, SLOT(updateColorScaleMax(double)));
    connect(this->spinBox_minDataColorScale, SIGNAL(valueChanged(double)),
            this->allSpectra, SLOT(updateColorScaleMin(double)));

    readSettings();

    this->recordPage->emit_defaultStorageDirectoryUpdated();

}

MainWindowUI::~MainWindowUI()
{
    this->writeSettings();
}

void MainWindowUI::buildWelcomePage()
{
    // LABEL
    label_serverTMEcho        = new QLabel(tr("TM Echo Server address"));
    label_serverTMEchoPort    = new QLabel(tr("TM Echo Server Port"));
    label_TMServerPort        = new QLabel(tr("TM Server Port:"));
    label_TCServerPort        = new QLabel(tr("TC Server Port:"));
    label_currentTCTMServerIP = new QLabel(tr("-"));
    label_totalOfBytes        = new QLabel(tr("Received Bytes: -"));
    label_totalOfPackets      = new QLabel(tr("Received Packets: -"));
    label_waveformScaling     = new QLabel(tr("wfrm scale factor [counts/V]: "));
    label_colorScale          = new QLabel(tr("color scale [dB]: "));
    // BUTTON
    button_clearConsole         = new QPushButton(tr("Clear console"));
    button_openSocketEchoServer = new QPushButton(tr("Open Echo Server"));
    button_resetStatistics      = new QPushButton(tr("Reset statistics"));
    button_openServerTCTM       = new QPushButton(tr("Open TC / TM Ports"));
    button_testServerTCTM       = new QPushButton(tr("Test TC / TM Ports"));
    button_TCAcknowledgement    = new QPushButton(tr("send TC Acknowledgement"));
    button_TCRejection          = new QPushButton(tr("send TC rejection"));
    button_GSEHK                = new QPushButton(tr("send GSE HK"));
    spinbox_TMServerPort        = new QSpinBox();
    spinbox_TCServerPort        = new QSpinBox();
    // GROUPBOX
    groupbox_TMEcho               = new QGroupBox(tr("TM Echo Server"));
    groupbox_ports                = new QGroupBox(tr("Ports for the TM and TC Servers"));
    groupbox_TMToForeignGSETester = new QGroupBox(tr("Send TM to the foreign GSE tester"));
    groupbox_scaleParameters      = new QGroupBox(tr("Parameters"));
    groupbox_console              = new QGroupBox(tr("Console"));
    // LAYOUT
    layout_TMEcho               = new QVBoxLayout();
    layout_ports                = new QGridLayout();
    layout_TMToForeignGSETester = new QVBoxLayout();
    layout_welcomePage          = new QGridLayout();
    layout_scalingFactor        = new QHBoxLayout();
    layout_colorMapScale        = new QHBoxLayout();
    layout_scaleParameters      = new QVBoxLayout();
    layout_console              = new QVBoxLayout();
    // SPINBOX
    spinBox_scaleFactor       = new QDoubleSpinBox();
    spinBox_maxDataColorScale = new QDoubleSpinBox();
    spinBox_minDataColorScale = new QDoubleSpinBox();
    spinbox_serverTMEchoPort  = new QSpinBox();
    // OTHER
    console                 = new QTextEdit();
    serverTMEchoDialogBox   = new QIPDialogBox();
    checkbox_displayASMInDB = new QCheckBox("display ASM in [dB]");
    displayLFRConfig        = new DisplayLFRConfig();

    label_waveformScaling->setToolTip(
                "this scale factor is used in the following tabs: SWF, CWF_F1, CWF_F2, CWF_F3 and FFT\nMINI-LFR 5800\nEM 9117"
                );
    label_colorScale->setToolTip("this value sets the color scale used in the FFT tab");

    spinBox_scaleFactor->setValue( 1. );
    spinBox_scaleFactor->setMaximum( 1e5 );
    spinBox_scaleFactor->setMinimum( 1e-3 );
    spinBox_maxDataColorScale->setMaximum( 1000 );
    spinBox_maxDataColorScale->setMinimum( -1000 );
    spinBox_minDataColorScale->setMaximum( 1000 );
    spinBox_minDataColorScale->setMinimum( -1000 );
    spinbox_TMServerPort->setRange(59000, 65535);
    spinbox_TCServerPort->setRange(59001, 65535);
    spinbox_serverTMEchoPort->setRange(59002, 65535);

    layout_scalingFactor->addWidget(label_waveformScaling);
    layout_scalingFactor->addWidget(spinBox_scaleFactor);

    layout_colorMapScale->addWidget( label_colorScale );
    layout_colorMapScale->addWidget( spinBox_minDataColorScale );
    layout_colorMapScale->addWidget( spinBox_maxDataColorScale );

    layout_scaleParameters->addLayout( layout_scalingFactor );
    layout_scaleParameters->addLayout( layout_colorMapScale );
    layout_scaleParameters->addWidget( checkbox_displayASMInDB );

    groupbox_scaleParameters->setLayout( layout_scaleParameters );

    serverTMEchoDialogBox->setIP(127, 0, 0, 1);

    layout_TMEcho->addWidget(label_serverTMEcho);
    layout_TMEcho->addWidget(serverTMEchoDialogBox);
    layout_TMEcho->addWidget(label_serverTMEchoPort);
    layout_TMEcho->addWidget(spinbox_serverTMEchoPort);
    layout_TMEcho->addWidget(button_openSocketEchoServer);
    layout_TMEcho->addWidget(label_totalOfPackets);
    layout_TMEcho->addWidget(label_totalOfBytes);
    layout_TMEcho->addWidget(button_resetStatistics);
    groupbox_TMEcho->setLayout(layout_TMEcho);

    layout_ports->addWidget(label_TMServerPort, 0, 0, 1, 1);
    layout_ports->addWidget(spinbox_TMServerPort, 0, 1, 1, 1);
    layout_ports->addWidget(label_TCServerPort, 1, 0, 1, 1);
    layout_ports->addWidget(spinbox_TCServerPort, 1, 1, 1, 1);
    layout_ports->addWidget(button_openServerTCTM, 2, 0, 1, 2);
    layout_ports->addWidget(button_testServerTCTM, 3, 0, 1, 2);
    groupbox_ports->setLayout(layout_ports);

    layout_TMToForeignGSETester->addWidget(button_TCAcknowledgement);
    layout_TMToForeignGSETester->addWidget(button_TCRejection);
    layout_TMToForeignGSETester->addWidget(button_GSEHK);
    groupbox_TMToForeignGSETester->setLayout(layout_TMToForeignGSETester);

    layout_console->addWidget( console );
    layout_console->addWidget( button_clearConsole );
    groupbox_console->setLayout(layout_console);

    layout_welcomePage->addWidget(groupbox_TMEcho, 0, 0, 1, 1);
    layout_welcomePage->addWidget(groupbox_console, 0, 1, 2, 1);
    layout_welcomePage->addWidget(groupbox_scaleParameters, 1, 0, 1, 1);
    layout_welcomePage->addWidget(displayLFRConfig, 2, 0, 2, 2);
    layout_welcomePage->setRowStretch(3, 1);
    layout_welcomePage->setColumnStretch(2, 1);
}

void MainWindowUI::setTCTMServerAddressIP(QString address)
{
    label_currentTCTMServerIP->setText(address);
}

QString MainWindowUI::getTMEchoServerAddress()
{
    return(serverTMEchoDialogBox->getIP());
}

int MainWindowUI::getTMPort()
{
    return spinbox_TMServerPort->value();
}

int MainWindowUI::getTCPort()
{
    return spinbox_TCServerPort->value();
}

void MainWindowUI::displayOnConsole(QString message)
{
    console->append(message);
}

void MainWindowUI::clearConsole()
{
    console->clear();
}

void MainWindowUI::totalOfBytesHasChanged(unsigned int nbBytes)
{
    label_totalOfBytes->setText(
                "Received Bytes: "
                + QString::number(nbBytes));
}

void MainWindowUI::totalOfPacketsHasChanged(unsigned int nbPackets)
{
    label_totalOfPackets->setText(
                "Received Packets: "
                + QString::number(nbPackets));
}

void MainWindowUI::readSettings()
{
    QSettings settings("lpp", "lfrsgse");
    spinBox_scaleFactor->setValue( settings.value( "scaleFactor", 1.0 ).toDouble() );
    spinBox_maxDataColorScale->setValue( settings.value( "maxDataColorScale", 1000).toDouble() );
    spinBox_minDataColorScale->setValue( settings.value( "minDataColorScale", 1e-3).toDouble() );
    checkbox_displayASMInDB->setChecked( settings.value( "displayASMInDB", false).toBool() );

}

void MainWindowUI::writeSettings()
{
    QSettings settings("lpp", "lfrsgse");
    settings.setValue("scaleFactor", spinBox_scaleFactor->value());
    settings.setValue("maxDataColorScale", spinBox_maxDataColorScale->value() );
    settings.setValue("minDataColorScale", spinBox_minDataColorScale->value() );
    settings.setValue("displayASMInDB", checkbox_displayASMInDB->isChecked() );
}

double MainWindowUI::getScaleFactor()
{
    return spinBox_scaleFactor->value();
}
