#ifndef MAINWINDOWUI_H
#define MAINWINDOWUI_H

#include <QWidget>
#include <qipdialogbox.h>
#include <QLabel>
#include <QPushButton>
#include <QGridLayout>
#include <QVBoxLayout>
#include <QGroupBox>
#include <QSpinBox>
#include <QTextEdit>
#include <QTabWidget>
#include <QCheckBox>

#include <wfdisplay.h>
#include <asmpage.h>
#include <parameterdump.h>

#include "tmstatistics.h"
#include "hkdisplay.h"
#include "recordpage.h"
#include "storehk.h"
#include <fsw_params_processing.h>

#include "spectrum2dplot.h"
#include "spectra.h"
#include "displaylfrconfig.h"

class MainWindowUI : public QWidget
{
    Q_OBJECT
public:
    explicit MainWindowUI(QWidget *parent = 0);
    ~MainWindowUI();

    void buildWelcomePage();
    void setTCTMServerAddressIP(QString address);
    QString getTMEchoServerAddress();
    int getTMPort();
    int getTCPort();
    void displayOnConsole(QString message);
    void readSettings();
    void writeSettings();
    double getScaleFactor();

    QTabWidget *spwTabWidget;
    QWidget* spwTabWidgetPage0;
    QWidget* spwTabWidgetPage1;
    QWidget* spwTabWidgetPage2;
    QWidget* spwTabWidgetPage3;

    TMStatistics* tmStatistics;

    WFDisplay* wfDisplay;
    WFPage* wfPage_CWF_F1;
    WFPage* wfPage_CWF_F2;
    WFPage* wfPage_CWF_F3;
    ASMPage* asmPage_F0;
    ASMPage* asmPage_F1;
    ASMPage* asmPage_F2;
    HKDisplay* hkDisplay;
    RecordPage* recordPage;

    StoreHK *storeHK;

    QWidget *widget_spectrogram;

    ParameterDump *parameterDump;

    spectra *allSpectra;

    DisplayLFRConfig *displayLFRConfig;

    QPushButton *button_openServerTCTM;
    QPushButton *button_testServerTCTM;
    QPushButton *button_openSocketEchoServer;
    QPushButton *button_TCAcknowledgement;
    QPushButton *button_TCRejection;
    QPushButton *button_GSEHK;
    QPushButton *button_clearConsole;
    QPushButton *button_resetStatistics;

    QIPDialogBox *serverTMEchoDialogBox;

    QSpinBox *spinbox_serverTMEchoPort;

    QCheckBox *checkbox_displayASMInDB;
    
signals:
    
public slots:
    void clearConsole();
    void totalOfBytesHasChanged(unsigned int nbBytes);
    void totalOfPacketsHasChanged(unsigned int nbPackets);

private:
    QLabel *label_serverTMEcho;
    QLabel *label_serverTMEchoPort;
    QLabel *label_TMServerPort;
    QLabel *label_TCServerPort;
    QLabel *label_currentTCTMServerIP;
    QLabel *label_totalOfBytes;
    QLabel *label_totalOfPackets;
    QLabel *label_waveformScaling;
    QLabel *label_colorScale;

    QTextEdit *console;

    QSpinBox *spinbox_TMServerPort;
    QSpinBox *spinbox_TCServerPort;

    QDoubleSpinBox *spinBox_scaleFactor;
    QDoubleSpinBox *spinBox_maxDataColorScale;
    QDoubleSpinBox *spinBox_minDataColorScale;

    QGroupBox *groupbox_TMEcho;
    QGroupBox *groupbox_ports;
    QGroupBox *groupbox_TMToForeignGSETester;
    QGroupBox *groupbox_scaleParameters;
    QGroupBox *groupbox_console;

    QVBoxLayout *layout_TMEcho;
    QVBoxLayout *layout_TMToForeignGSETester;
    QVBoxLayout *layout_overallLayout;
    QHBoxLayout *layout_scalingFactor;
    QHBoxLayout *layout_colorMapScale;
    QVBoxLayout *layout_scaleParameters;
    QVBoxLayout *layout_console;

    QGridLayout *layout_welcomePage;
    QGridLayout *layout_ports;
    
};

#endif // MAINWINDOWUI_H
