#include "spectrum2dplot.h"

#include "stdio.h"
#include <math.h>

Spectrum2DPlot::Spectrum2DPlot(QWidget *parent, int keySize, int valueSize, double scaleFactor) :
    QWidget(parent)
{
    this->keySize = keySize;
    this->valueSize = valueSize;
    this->scaleFactor = scaleFactor;

    //************
    //************
    mainLayout = new QGridLayout( this );
    hbox_colorMapScale = new QHBoxLayout( );
    customPlot = new QCustomPlot( );

    wfPlot = new WFPlot( 0, 2688, 32000);

    wfrmLayout = new QGridLayout( );
    fftLayout = new QGridLayout( );
    hbox_wfrmPlot = new QHBoxLayout( );
    hbox_meanStandardDeviation = new QHBoxLayout( );
    wfPlot2 = new WFPlot( 0, 2688, 32000);
    //************
    //************

    button_autoScale = new QPushButton( tr("AutoScale") );

    colorScale = new QCPColorScale( customPlot );
    colorMap = new QCPColorMap(customPlot->xAxis, customPlot->yAxis);
    group = new QCPMarginGroup(customPlot);
    spinBox_maxDataColorScale = new QDoubleSpinBox();
    spinBox_minDataColorScale = new QDoubleSpinBox();
    radio_rect = new QRadioButton(tr("rect"));
    radio_hamming = new QRadioButton(tr("Hamming"));
    radio_hann = new QRadioButton(tr("Hann"));

    label_colorScale = new QLabel(tr("set color scale: "));
    label_average = new QLabel(tr("mean = "));
    label_standardDeviation = new QLabel(tr("std dev = "));

    colorMap->data()->setSize(this->keySize, this->valueSize);
    colorMap->data()->setRange(QCPRange(0, this->keySize), QCPRange(0, 2048));
    for (unsigned int x=0; x<this->keySize; ++x)
      for (unsigned int y=0; y<this->valueSize; ++y)
        colorMap->data()->setCell(x, y, 1.0);
    colorMap->setGradient(QCPColorGradient::gpJet);
    colorMap->setInterpolate( false );
    colorMap->rescaleDataRange( true );

    colorScale->setMarginGroup(QCP::msTop|QCP::msBottom, group);
    colorScale->setGradient(QCPColorGradient::gpJet);
    colorScale->setDataRange( colorMap->dataRange() );

    maxDataColorScale = colorMap->data()->dataBounds().maxRange;
    minDataColorScale = colorMap->data()->dataBounds().minRange;
    spinBox_maxDataColorScale->setValue( maxDataColorScale );
    spinBox_minDataColorScale->setValue( minDataColorScale );
    spinBox_maxDataColorScale->setMaximum( 1000 );
    spinBox_maxDataColorScale->setMinimum( -1000 );
    spinBox_minDataColorScale->setMaximum( 1000 );
    spinBox_minDataColorScale->setMinimum( -1000 );

    customPlot->addPlottable(colorMap);
    customPlot->plotLayout()->addElement(0, 1, colorScale);
    customPlot->axisRect()->setMarginGroup(QCP::msTop|QCP::msBottom, group);
    customPlot->rescaleAxes();
    customPlot->replot();

    //*****
    // FFTW
    fftsize     = (valueSize-1) * 2;
    fft_input   = fftw_alloc_real( fftsize );
    fft_rect    = fftw_alloc_real( fftsize );
    fft_hamming = fftw_alloc_real( fftsize );
    fft_hann    = fftw_alloc_real( fftsize );
    fft_output  = fftw_alloc_complex( fftsize / 2 + 1 );
    fftplan     = fftw_plan_dft_r2c_1d( fftsize, fft_input, fft_output, FFTW_MEASURE);

   for(unsigned int n=0; n<fftsize; ++n)
     fft_hamming[n] = 0.54 - 0.46*cos(2 * M_PI * n / fftsize);
   for(unsigned int n=0; n<fftsize; ++n)
     fft_hann[n] = 0.5 - 0.5*cos(2 * M_PI * n / fftsize);
   for(unsigned int n=0; n<fftsize; ++n)
     fft_hann[n] = 0.5 - 0.5*cos(2 * M_PI * n / fftsize);

   fft_window = fft_rect;

   radio_rect->toggle();
    //
    //*****

    hbox_colorMapScale->addWidget( label_colorScale );
    hbox_colorMapScale->addWidget( spinBox_minDataColorScale );
    hbox_colorMapScale->addWidget( spinBox_maxDataColorScale );
    hbox_colorMapScale->addWidget( button_autoScale );
    hbox_colorMapScale->addStretch();

    hbox_wfrmPlot->addWidget( radio_rect );
    hbox_wfrmPlot->addWidget( radio_hamming );
    hbox_wfrmPlot->addWidget( radio_hann );
    hbox_wfrmPlot->addStretch();

    wfrmLayout->addLayout( hbox_wfrmPlot, 0, 0, 1, 2);
    wfrmLayout->addWidget( wfPlot2, 1, 0, 1, 2);

    hbox_meanStandardDeviation->addWidget(label_average);
    hbox_meanStandardDeviation->addWidget(label_standardDeviation);
    hbox_meanStandardDeviation->addStretch();

    fftLayout->addLayout( hbox_meanStandardDeviation, 0, 0, 1, 2);
    fftLayout->addWidget( wfPlot, 1, 0, 1, 2);

    mainLayout->addLayout( hbox_colorMapScale, 0, 0, 1, 4);
    mainLayout->addWidget( customPlot, 1, 0, 2, 4);
    mainLayout->addLayout( fftLayout, 3, 0, 1, 2);
    mainLayout->addLayout( wfrmLayout, 3, 2, 1, 2);

    this->setLayout( mainLayout );

    connect(this->button_autoScale, SIGNAL(clicked()),
            this, SLOT(autoScale()) );
    connect(this->spinBox_maxDataColorScale, SIGNAL(valueChanged(double)),
            this, SLOT(valueMaxChanged(double)));
    connect(this->spinBox_minDataColorScale, SIGNAL(valueChanged(double)),
            this, SLOT(valueMinChanged(double)));
    connect(this->radio_hamming, SIGNAL(toggled(bool)),
            this, SLOT(setHamming()));
    connect(this->radio_hann, SIGNAL(toggled(bool)),
            this, SLOT(setHann()));
}

void Spectrum2DPlot::addData( void )
{
    QCPColorMapData *data;
    unsigned int key;
    unsigned int value;

    data = new QCPColorMapData( keySize, valueSize,
                            colorMap->data()->keyRange(), colorMap->data()->valueRange());

    for ( key=0; key<keySize-1; ++key)
      for ( value=0; value<valueSize; ++value)
          data->setCell( key+1, value, colorMap->data()->cell( key, value ) );

    key = 0;
    for ( value=0; value<valueSize; ++value)
        data->setCell( key+1, value, 0 );

    for ( key=0; key<keySize; ++key)
      for ( value=0; value<valueSize; ++value)
          colorMap->data()->setCell( key, value, data->cell( key, value ) );

    customPlot->replot();
}

void Spectrum2DPlot::cleanValues( void )
{
    unsigned int key;
    unsigned int value;

    for ( key=0; key<keySize; ++key)
      for ( value=0; value<valueSize; ++value)
          colorMap->data()->setCell( key, value, 0.0 );
}

void Spectrum2DPlot::autoScale( void )
{
    colorMap->rescaleDataRange( true );
    colorScale->setDataRange( colorMap->dataRange() );
    colorScale->rescaleDataRange( false );
    customPlot->replot();
    spinBox_maxDataColorScale->setValue( colorMap->data()->dataBounds().upper );
    spinBox_minDataColorScale->setValue( colorMap->data()->dataBounds().lower );
}

void Spectrum2DPlot::updateSpinBoxMaxDataColorScale(double newValue)
{
    spinBox_maxDataColorScale->setValue( newValue );
}

void Spectrum2DPlot::updateSpinBoxMinDataColorScale(double newValue)
{
    spinBox_minDataColorScale->setValue( newValue );
}

void Spectrum2DPlot::valueMaxChanged(double newValue)
{
    maxDataColorScale = newValue;
    updateColorMapAndColorScale( QCPRange( minDataColorScale, maxDataColorScale ) );
}

void Spectrum2DPlot::valueMinChanged(double newValue)
{
    minDataColorScale = newValue;
    updateColorMapAndColorScale( QCPRange( minDataColorScale, maxDataColorScale ) );
}

void Spectrum2DPlot::updateScaleFactor(double scaleFactor)
{
    this->scaleFactor = scaleFactor;
}

void Spectrum2DPlot::updateColorMapAndColorScale( QCPRange range )
{
    colorMap->setDataRange( range );
//    colorMap->rescaleDataRange( false );
    colorScale->setDataRange( colorMap->dataRange() );
    colorScale->rescaleDataRange( false );
    customPlot->replot();
}

void Spectrum2DPlot::computeFFTAndDisplay( short *dataBuffer, unsigned int bufferSize )
{
    unsigned int key;
    unsigned int value;
    unsigned int k;
    double average;
    double spectrum[ bufferSize/2 + 1 ];
    double fAxis[ bufferSize/2 + 1 ];

    if ( fftsize != bufferSize)
    {
        printf("Error in computeFFTAndDisplay, wrong bufferSize\n");
        exit(0);
    }

    for (k = 0; k<fftsize; k++)
    {
        fAxis[k] = k * 4096./2688.;
    }

    if ( scaleFactor != 1)
    {
        for (k = 0; k<fftsize; k++)
            this->fft_input[k] = dataBuffer[k] / scaleFactor;
    }
    else
    {
        for (k = 0; k<fftsize; k++)
            this->fft_input[k] = dataBuffer[k];
    }

    average = computeAverage( fft_input, fftsize );
    computeStandardDeviation( computeVariance( fft_input, fftsize, average ) );

    if ( !radio_rect->isChecked() )
    {
#pragma omp parallel for
        for(k=0; k<fftsize; k++)
            fft_input[k] = fft_input[k]*fft_window[k];
    }

    fftw_execute(this->fftplan);

    spectrum[0] = 20 * log10(
                sqrt( fft_output[0][0] * fft_output[0][0] + fft_output[0][1] * fft_output[0][1] )
            / ((double) fftsize)
            );

    for (k = 1; k<(fftsize/2 + 1); k++)
    {
        spectrum[k] = 20 * log10(
                    sqrt( fft_output[k][0] * fft_output[k][0] + fft_output[k][1] * fft_output[k][1] )
                / ((double) fftsize)
                );
    }

    // shift the data in the current colorMap
    for ( key=0; key<keySize-1; key++ )
        for ( value=0; value<valueSize; value++ )
            colorMap->data()->setCell( keySize - key - 1, value,
                                       colorMap->data()->cell( keySize - key - 2, value )
                                       );

    for ( value=0; value<valueSize; value++)
        colorMap->data()->setCell( 0, value, spectrum[value] );

    wfPlot->displayOnPlotDouble( spectrum, fAxis, bufferSize/2+1 );
    wfPlot2->displayOnPlotDouble( fft_input, NULL, fftsize );

    customPlot->replot();
}

void Spectrum2DPlot::setHamming()
{
    fft_window = fft_hamming;
}

void Spectrum2DPlot::setHann()
{
    fft_window = fft_hann;
}

void Spectrum2DPlot::setRect()
{
    fft_window = fft_rect;
}

double Spectrum2DPlot::computeAverage(double *dataBuffer, unsigned int nbData)
{
    unsigned int i;
    double sum;
    double average;

    sum = 0.;

    for (i = 0; i < nbData; i++)
    {
        sum = sum + dataBuffer[i];
    }

    average = sum / nbData;

    label_average->setText("mean = " + QString::number(average, 'e', 3));

    return average;
}

double Spectrum2DPlot::computeVariance(double *dataBuffer, unsigned int nbData, double average)
{
    unsigned int i;
    double sum;
    double variance;

    sum = 0.;

    for (i = 0; i < nbData; i++)
    {
        sum = sum + (dataBuffer[i] - average) * (dataBuffer[i] - average);
    }

    variance = sum / nbData;

    return variance;
}

double Spectrum2DPlot::computeStandardDeviation(double variance)
{
    double standardDeviation;

    standardDeviation = sqrt(variance);

    label_standardDeviation->setText("std dev = " + QString::number(standardDeviation, 'e', 3) );

    return standardDeviation;
}
