#ifndef SPECTRUM2DPLOT_H
#define SPECTRUM2DPLOT_H

#include <QWidget>
#include <QGridLayout>
#include <QHBoxLayout>
#include <QPushButton>
#include <qcustomplot.h>
#include <QSpinBox>
#include <QSettings>
#include <QLabel>
#include <QCheckBox>
#include <QRadioButton>
#include <QTableWidget>

#include <wfplot.h>

#include "fftw3.h"

class Spectrum2DPlot : public QWidget
{
    Q_OBJECT
public:
    explicit Spectrum2DPlot(QWidget *parent = 0, int keySize = 0, int valueSize = 0 , double scaleFactor = 1.0);

    void updateColorMapAndColorScale( QCPRange range );
    double computeAverage( double *dataBuffer, unsigned int nbData );
    double computeVariance(double *dataBuffer, unsigned int nbData , double average);
    double computeStandardDeviation( double variance );

    QCPColorMap *colorMap;
    QCustomPlot *customPlot;
    QCPColorScale *colorScale;
    QCPMarginGroup *group;
    QGridLayout *mainLayout;
    QGridLayout *wfrmLayout;
    QGridLayout *fftLayout;
    QHBoxLayout *hbox_colorMapScale;
    QHBoxLayout *hbox_wfrmPlot;
    QHBoxLayout *hbox_meanStandardDeviation;

    QLabel *label_colorScale;
    QLabel *label_average;
    QLabel *label_standardDeviation;

    double maxDataColorScale;
    double minDataColorScale;
    QDoubleSpinBox *spinBox_maxDataColorScale;
    QDoubleSpinBox *spinBox_minDataColorScale;
    QPushButton *button_autoScale;

    QRadioButton * radio_rect;
    QRadioButton * radio_hamming;
    QRadioButton * radio_hann;

    unsigned int keySize;
    unsigned int valueSize;
    double scaleFactor;

    WFPlot *wfPlot;
    WFPlot *wfPlot2;

    fftw_plan fftplan;
    double *fft_input;
    double* fft_window;
    double* fft_rect;
    double* fft_hamming;
    double* fft_hann;
    fftw_complex *fft_output;
    unsigned int fftsize;

signals:

public slots:
    void addData( void );
    void cleanValues( void );
    void autoScale( void );
    void updateSpinBoxMaxDataColorScale(double newValue);
    void updateSpinBoxMinDataColorScale(double newValue);
    void computeFFTAndDisplay( short *dataBuffer, unsigned int bufferSize );
    void valueMaxChanged( double newValue);
    void valueMinChanged( double newvalue);
    void updateScaleFactor(double scaleFactor);
    void setHamming( void );
    void setHann( void );
    void setRect( void );

};

#endif // SPECTRUM2DPLOT_H
