#include "storehk.h"

StoreHK::StoreHK(QObject *parent) :
    QObject(parent)
{
    logFile = new QFile();
}

void StoreHK::buildDataAsQByteArray(short temp_scm, short temp_pcb, short temp_fpga,
                                    short param1, short param2, short param3,
                                    unsigned int coarseTime, unsigned int fineTime )
{
    double sampleTime;
    QByteArray sampleTimeQByteArray;

    initDataBuffer();
    sampleTime = 0;

    if (storageEnabled == true)
    {
        sampleTime = (double) (coarseTime)
                + ((double) (fineTime)) * 1. / 65536.;
        sampleTimeQByteArray.setNum( sampleTime, 'f', 10 );

        dataBuffer.append( sampleTimeQByteArray + ' '
                           + QByteArray::number(temp_scm) + ' '
                           + QByteArray::number(temp_pcb) + ' '
                           + QByteArray::number(temp_fpga) + ' '
                           + QByteArray::number(param1) + ' '
                           + QByteArray::number(param2) + ' '
                           + QByteArray::number(param3) );
        storeDataBuffer();
    }
}

QString StoreHK::buildPrefix()
{
    QTime time;
    QDate date;
    QString dateTime;
    QString prefix;

    date = QDate::currentDate();
    time = QTime::currentTime();

    dateTime =
            QString("%1").arg(date.year(),    4, 10, QChar('0')) + "_"
            + QString("%1").arg(date.month(), 2, 10, QChar('0')) + "_"
            + QString("%1").arg(date.day(),   2, 10, QChar('0')) + "_"
            + QString("%1").arg(time.hour(),  2, 10, QChar('0')) + "_"
            + QString("%1").arg(time.minute(),2, 10, QChar('0')) + "_"
            + QString("%1").arg(time.second(),2, 10, QChar('0'));

    prefix = defaultStorageDirectory + "/" + dateTime + "_" + hkName ;

    return prefix;
}

void StoreHK::buildFileName()
{
    QString prefix;

    prefix = buildPrefix() ;

    if(this->logFile->isOpen()) this->logFile->close();
    this->logFile->setFileName( prefix + ".data");
    if(this->logFile->open(QIODevice::WriteOnly)) this->logFileStrm = new QTextStream(this->logFile);

    *(this->logFileStrm) << "time temp_scm temp_pcb temp_fpga v e1 e2" << endl;

    storageEnabled = true;
}

void StoreHK::storeDataBuffer()
{
    *(this->logFileStrm) << dataBuffer << endl;
    if (storageEnabled == false){
        this->logFileStrm->flush();
        this->logFile->waitForBytesWritten(3000);
        this->logFile->close();
    }
}

void StoreHK::storeData()
{
    if (logFileEn == false)
    {
        buildFileName();
        logFileEn = true;
    }
    else
    {
        // disable storage
        storageEnabled = false;
        logFileEn = false;
    }
}

void StoreHK::initDataBuffer()
{
    dataBuffer.clear();
}

void StoreHK::setDefaultStorageDirectory(QString nameOfTheDirectory)
{
    defaultStorageDirectory = nameOfTheDirectory;
}
